@@ -2,8 +2,9 @@
 # Copyright (c) 2008, Riverbank Computing Limited
 # All rights reserved.
 #
-# This software is provided without warranty under the terms of the GPL v2
-# license.
+# This software is provided without warranty under the terms of the BSD license.
+# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply
+
 #
 # Author: Riverbank Computing Limited
 #------------------------------------------------------------------------------
@@ -182,6 +183,7 @@ class SimpleEditor ( BaseEditor ):
         super( SimpleEditor, self ).init( parent )
         
         self.control = control = QtGui.QComboBox()
+        control.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
         control.addItems(self.names)
         QtCore.QObject.connect(control,
                 QtCore.SIGNAL('currentIndexChanged(QString)'),
@@ -247,12 +249,12 @@ class SimpleEditor ( BaseEditor ):
                     idx = self.control.findText(self.inverse_mapping[self.value])
                     self.control.setCurrentIndex(idx)
                 except:
-                    pass
+                    self.control.setCurrentIndex(-1)
             else:
                 try:
                     self.control.setEditText(self.str_value)
                 except:
-                    pass
+                    self.control.setEditText('')
         
     #---------------------------------------------------------------------------
     #  Handles an error that occurs while setting the object's trait value:
