@@ -2,8 +2,9 @@
 # Copyright (c) 2007, Riverbank Computing Limited
 # All rights reserved.
 #
-# This software is provided without warranty under the terms of the GPL v2
-# license.
+# This software is provided without warranty under the terms of the BSD license.
+# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply
+
 #
 # Author: Riverbank Computing Limited
 #------------------------------------------------------------------------------
@@ -17,26 +18,20 @@
 
 from PyQt4 import QtCore, QtGui
 
-from enthought.traits.api \
-    import Str, Any, Bool
+from enthought.pyface.api import ImageResource
 
-from enthought.traits.trait_base \
-    import user_name_for, enumerate, xgetattr
+from enthought.traits.api import Str, Any, Bool
+from enthought.traits.trait_base import user_name_for, enumerate, xgetattr
     
 # FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
 # compatibility. The class has been moved to the 
 # enthought.traits.ui.editors.list_editor file.
-from enthought.traits.ui.editors.list_editor \
-    import ListItemProxy, ToolkitEditorFactory
+from enthought.traits.ui.editors.list_editor import ListItemProxy, \
+    ToolkitEditorFactory
 
-from editor \
-    import Editor
-
-from helper \
-    import IconButton
-
-from menu \
-    import MakeMenu
+from editor import Editor
+from helper import IconButton
+from menu import MakeMenu
 
 #-------------------------------------------------------------------------------
 #  'SimpleEditor' class:
@@ -108,10 +103,14 @@ class SimpleEditor ( Editor ):
         # Create a scrolled window to hold all of the list item controls:
         self.control = QtGui.QScrollArea()
         self.control.setFrameShape(QtGui.QFrame.NoFrame)
+        self.control.setWidgetResizable(True)
 
         # Create a widget with a grid layout as the container.
         self._list_pane = QtGui.QWidget()
+        self._list_pane.setSizePolicy(QtGui.QSizePolicy.Expanding,
+                                      QtGui.QSizePolicy.Expanding)
         layout = QtGui.QGridLayout(self._list_pane)
+        layout.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)
         layout.setMargin(0)
 
         # Remember the editor to use for each individual list item:
@@ -203,10 +202,6 @@ class SimpleEditor ( Editor ):
         else:
             rows = self.factory.rows
 
-        #list_pane.SetSize( wx.Size(
-        #     width + ((trait_handler.maxlen > rows) * scrollbar_dx),
-        #     height * rows ) )
-
         # QScrollArea can have problems if the widget being scrolled is set too
         # early (ie. before it contains something).
         if self.control.widget() is None:
@@ -522,8 +517,19 @@ class NotebookEditor ( Editor ):
 
         # Create a tab widget to hold each separate object's view:
         self.control = QtGui.QTabWidget()
-        QtCore.QObject.connect(self.control,
-                QtCore.SIGNAL('currentChanged(int)'), self._tab_activated)
+        signal = QtCore.SIGNAL( 'currentChanged(int)' )
+        QtCore.QObject.connect( self.control, signal, self._tab_activated )
+
+        # Create the button to close tabs, if necessary:
+        if self.factory.deletable:
+            button = QtGui.QToolButton()
+            button.setAutoRaise( True )
+            button.setToolTip( 'Remove current tab ')
+            button.setIcon ( ImageResource( 'closetab' ).create_icon() )
+
+            self.control.setCornerWidget( button, QtCore.Qt.TopRightCorner )
+            signal = QtCore.SIGNAL( 'clicked()' )
+            QtCore.QObject.connect( button, signal, self.close_current )
 
         # Set up the additional 'list items changed' event handler needed for
         # a list based trait:
@@ -546,10 +552,10 @@ class NotebookEditor ( Editor ):
 
         # Create a tab page for each object in the trait's value:
         for object in self.value:
-            page, view_object, monitoring = self._create_page(object)
+            ui, view_object, monitoring = self._create_page(object)
 
             # Remember the page for later deletion processing:
-            self._uis.append([page, object, view_object, monitoring])
+            self._uis.append([ui.control, ui, view_object, monitoring])
 
     #---------------------------------------------------------------------------
     #  Handles some subset of the trait's list being updated:
@@ -564,11 +570,11 @@ class NotebookEditor ( Editor ):
         page_name = self.factory.page_name[1:]
 
         for i in event.removed:
-            page, _, view_object, monitoring = self._uis[index]
+            page, ui, view_object, monitoring = self._uis[index]
             if monitoring:
                 view_object.on_trait_change(self.update_page_name, page_name,
                         remove=True)
-
+            ui.dispose()
             self.control.removeTab(self.control.indexOf(page))
 
             del self._uis[index]
@@ -576,16 +582,31 @@ class NotebookEditor ( Editor ):
         # Add a page for each added object:
         first_page = None
         for object in event.added:
-            page, view_object, monitoring  = self._create_page(object)
-            self._uis[index:index] = [[page, object, view_object, monitoring]]
+            ui, view_object, monitoring  = self._create_page(object)
+            self._uis[index:index] = [[ui.control, ui, view_object, monitoring]]
             index += 1
 
             if first_page is None:
-                first_page = page
+                first_page = ui.control
 
         if first_page is not None:
             self.control.setCurrentWidget(first_page)
 
+    #---------------------------------------------------------------------------
+    #  Closes the currently selected tab:
+    #---------------------------------------------------------------------------
+                                    
+    def close_current ( self, force=False ):
+        """ Closes the currently selected tab:
+        """
+        widget = self.control.currentWidget()
+        for i in xrange( len( self._uis ) ):
+            page, ui, _, _ = self._uis[i]
+            if page is widget:
+                if force or ui.handler.close( ui.info, True ):
+                    del self.value[i]
+                break
+
     #---------------------------------------------------------------------------
     #  Closes all currently open notebook pages:
     #---------------------------------------------------------------------------
@@ -595,10 +616,11 @@ class NotebookEditor ( Editor ):
         """
         page_name = self.factory.page_name[1:]
 
-        for _, _, view_object, monitoring in self._uis:
+        for _, ui, view_object, monitoring in self._uis:
             if monitoring:
                 view_object.on_trait_change(self.update_page_name, page_name,
                         remove=True)
+            ui.dispose()
 
         # Reset the list of ui's and dictionary of page name counts:
         self._uis = []
@@ -627,8 +649,8 @@ class NotebookEditor ( Editor ):
         """ Handles the trait defining a particular page's name being changed.
         """
         for i, value in enumerate(self._uis):
-            page, user_object, _, _ = value
-            if object is user_object:
+            page, ui, _, _ = value
+            if object is ui.info.object:
                 name = None
                 handler = getattr(self.ui.handler,
                         '%s_%s_page_name' % (self.object_name, self.name),
@@ -702,7 +724,7 @@ class NotebookEditor ( Editor ):
         else:
             self.control.addTab(ui.control, image, name)
 
-        return (ui.control, view_object, monitoring)
+        return (ui, view_object, monitoring)                        
 
     #---------------------------------------------------------------------------
     #  Handles a notebook tab being 'activated' (i.e. clicked on) by the user:
@@ -712,11 +734,10 @@ class NotebookEditor ( Editor ):
         """ Handles a notebook tab being "activated" (i.e. clicked on) by the
             user.
         """
-        w = self.control.widget(idx)
-
-        for page, object, _, _ in self._uis:
-            if page is w:
-                self.selected = object
+        widget = self.control.widget(idx)
+        for page, ui, _, _ in self._uis:
+            if page is widget:
+                self.selected = ui.info.object
                 break
 
     #---------------------------------------------------------------------------
@@ -726,7 +747,7 @@ class NotebookEditor ( Editor ):
     def _selected_changed(self, selected):
         """ Handles the **selected** trait being changed.
         """
-        for page, object, _, _ in self._uis:
-            if selected is object:
+        for page, ui, _, _ in self._uis:
+            if selected is ui.info.object:
                 self.control.setCurrentWidget(page)
                 break
