@@ -2,8 +2,9 @@
 # Copyright (c) 2008, Riverbank Computing Limited
 # All rights reserved.
 #
-# This software is provided without warranty under the terms of the GPL v2
-# license.
+# This software is provided without warranty under the terms of the BSD license.
+# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply
+
 #
 # Author: Riverbank Computing Limited
 #------------------------------------------------------------------------------
@@ -66,6 +67,9 @@ class SimpleSliderEditor ( Editor ):
     # Formatting string used to format value and labels
     format = Str
 
+    # Function to evaluate textual user input
+    evaluate = Any
+
     #---------------------------------------------------------------------------
     #  Finishes initializing the editor by creating the underlying toolkit
     #  widget:
@@ -84,13 +88,14 @@ class SimpleSliderEditor ( Editor ):
 
         self.format = factory.format
 
+        self.evaluate = factory.evaluate
+        self.sync_value( factory.evaluate_name, 'evaluate', 'from' )
+
         if factory.label_width > 0:
             size = wx.Size( factory.label_width, 20 )
 
         self.sync_value( factory.low_name,  'low',  'from' )
         self.sync_value( factory.high_name, 'high', 'from' )
-        low  = self.low
-        high = self.high
 
         # The control is a horizontal layout.
         self.control = panel = QtGui.QHBoxLayout()
@@ -99,18 +104,12 @@ class SimpleSliderEditor ( Editor ):
 
         try:
             fvalue_text = self.format % fvalue
-            1 / (low <= fvalue <= high)
+            1 / (self.low <= fvalue <= self.high)
         except:
             fvalue_text = ''
             fvalue      = low
 
-        if high > low:
-            ivalue = int( (float( fvalue - low ) / (high - low)) * 10000 )
-        else:
-            ivalue = low
-
-            if ivalue is None:
-                ivalue = 0
+        ivalue = self._convert_to_slider(fvalue)
 
         self._label_lo = QtGui.QLabel()
         self._label_lo.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
@@ -164,11 +163,7 @@ class SimpleSliderEditor ( Editor ):
     def update_object_on_scroll(self, pos):
         """ Handles the user changing the current slider value.
         """
-        value = self.low + ((float(pos) / 10000.0) * (self.high - self.low))
-
-        if not self.factory.is_float:
-            value = int(value)
-
+        value = self._convert_from_slider(pos)
         self.control.text.setText(self.format % value)
         self.value = value
 
@@ -181,7 +176,7 @@ class SimpleSliderEditor ( Editor ):
         """
         try:
             try:
-                value = eval(unicode(self.control.text.text()).strip())
+                value = self.evaluate(unicode(self.control.text.text()).strip())
             except Exception, ex:
                 # The entered something that didn't eval as a number, (e.g.,
                 # 'foo') pretend it didn't happen
@@ -192,9 +187,7 @@ class SimpleSliderEditor ( Editor ):
                 value = int(value)
 
             self.value = value
-            self.control.slider.setValue(
-                int( ((float( value ) - self.low) /
-                     (self.high - self.low)) * 10000 ) )
+            self.control.slider.setValue(self._convert_to_slider(self.value))
         except TraitError, excp:
             pass
 
@@ -216,13 +209,7 @@ class SimpleSliderEditor ( Editor ):
             text  = ''
             value = low
 
-        if high > low:
-            ivalue = int( (float( value - low ) / (high - low)) * 10000.0 )
-        else:
-            ivalue = low
-
-            if ivalue is None:
-                ivalue = 0
+        ivalue = self._convert_to_slider(value)
 
         self.control.text.setText(text)
 
@@ -265,6 +252,56 @@ class SimpleSliderEditor ( Editor ):
             self._label_hi.setText(self.format % high)
             self.update_editor()
 
+    def _convert_to_slider(self, value):
+        """ Returns the slider setting corresponding to the user-supplied value.
+        """
+        if self.high > self.low:
+            ivalue = int( (float( value - self.low ) / 
+                           (self.high - self.low)) * 10000.0 )
+        else:
+            ivalue = self.low
+
+            if ivalue is None:
+                ivalue = 0
+        return ivalue
+
+    def _convert_from_slider(self, ivalue):
+        """ Returns the float or integer value corresponding to the slider 
+        setting.
+        """
+        value = self.low + ((float( ivalue ) / 10000.0) *
+                            (self.high - self.low))
+        if not self.factory.is_float:
+            value = int(round(value))
+        return value
+    
+
+#-------------------------------------------------------------------------------
+class LogRangeSliderEditor ( SimpleSliderEditor ):
+#-------------------------------------------------------------------------------
+    """ A slider editor for log-spaced values 
+    """
+
+    def _convert_to_slider(self, value):
+        """ Returns the slider setting corresponding to the user-supplied value.
+        """
+        value = max(value, self.low)
+        ivalue = int( (log10(value) - log10(self.low)) /
+                      (log10(self.high) - log10(self.low)) * 10000.0)
+        return ivalue
+
+    def _convert_from_slider(self, ivalue):
+        """ Returns the float or integer value corresponding to the slider 
+        setting.
+        """
+        value = float( ivalue ) / 10000.0 * (log10(self.high) -log10(self.low))
+        # Do this to handle floating point errors, where fvalue may exceed 
+        # self.high.
+        fvalue = min(self.low*10**(value), self.high)
+        if not self.factory.is_float:
+            fvalue = int(round(fvalue))
+        return fvalue
+
 #-------------------------------------------------------------------------------
 #  'LargeRangeSliderEditor' class:
 #-------------------------------------------------------------------------------
@@ -292,6 +329,9 @@ class LargeRangeSliderEditor ( Editor ):
     # High end of displayed range
     cur_high = Float
 
+    # Function to evaluate textual user input
+    evaluate = Any
+
     # Flag indicating that the UI is in the process of being updated
     ui_changing = Bool(False)
 
@@ -309,10 +349,12 @@ class LargeRangeSliderEditor ( Editor ):
         # Initialize using the factory range defaults:
         self.low = factory.low
         self.high = factory.high
+        self.evaluate = factory.evaluate
 
         # Hook up the traits to listen to the object.
         self.sync_value( factory.low_name,  'low',  'from' )
         self.sync_value( factory.high_name, 'high', 'from' )
+        self.sync_value( factory.evaluate_name, 'evaluate', 'from' )
 
         self.init_range()
         low  = self.cur_low
@@ -414,7 +456,7 @@ class LargeRangeSliderEditor ( Editor ):
         """ Handles the user pressing the Enter key in the text field.
         """
         try:
-            self.value = eval(unicode(self.control.text.text()).strip())
+            self.value = self.evaluate(unicode(self.control.text.text()).strip())
         except TraitError, excp:
             pass
 
@@ -731,7 +773,8 @@ SimpleEditorMap = {
     'xslider': LargeRangeSliderEditor,
     'spinner': SimpleSpinEditor,
     'enum':    SimpleEnumEditor,
-    'text':    RangeTextEditor
+    'text':    RangeTextEditor,
+    'low':     LogRangeSliderEditor
 }
 # Mapping between editor factory modes and custom editor classes
 CustomEditorMap = {
@@ -739,5 +782,6 @@ CustomEditorMap = {
     'xslider': LargeRangeSliderEditor,
     'spinner': SimpleSpinEditor,
     'enum':    CustomEnumEditor,
-    'text':    RangeTextEditor
+    'text':    RangeTextEditor,
+    'low':     LogRangeSliderEditor
 }
