@@ -2,8 +2,9 @@
 # Copyright (c) 2007, Riverbank Computing Limited
 # All rights reserved.
 #
-# This software is provided without warranty under the terms of the GPL v2
-# license.
+# This software is provided without warranty under the terms of the BSD license.
+# However, when used with the GPL version of PyQt the additional terms described
+# in the PyQt GPL exception also apply.
 #
 # Author: Riverbank Computing Limited
 #------------------------------------------------------------------------------
@@ -18,18 +19,22 @@ the PyQt user interface toolkit.
 
 from PyQt4 import QtCore, QtGui
 
-# Check the version numbers are late enough.
+# Check the version numbers are late enough:
 if QtCore.QT_VERSION < 0x040200:
     raise RuntimeError, "Need Qt v4.2 or higher, but got v%s" % QtCore.QT_VERSION_STR
 
-if QtCore.PYQT_VERSION < 0x040100:
-    raise RuntimeError, "Need PyQt v4.1 or higher, but got v%s" % QtCore.PYQT_VERSION_STR
-
 # Make sure a QApplication object is created early:
+import sys
 if QtGui.QApplication.startingUp():
-    import sys
     _app = QtGui.QApplication(sys.argv)
 
+# Make sure that SIGINTs actually stop the application event loop:
+import signal
+def _handler(signum, frame):
+    QtGui.QApplication.instance().quit()
+    sys.exit(signum)
+signal.signal(signal.SIGINT, _handler)
+
 from enthought.traits.trait_notifiers import set_ui_handler
 
 from enthought.traits.ui.toolkit import Toolkit
@@ -41,6 +46,8 @@ from constants import screen_dx, screen_dy
 #  the UI thread:
 #-------------------------------------------------------------------------------
 
+_QT_TRAITS_EVENT = QtCore.QEvent.Type(QtCore.QEvent.registerEventType())
+
 class _CallAfter(QtCore.QObject):
     """ This class dispatches a handler so that it executes in the main GUI
         thread (similar to the wx function).
@@ -67,20 +74,35 @@ class _CallAfter(QtCore.QObject):
 
         # Move to the main GUI thread.
         self.moveToThread(QtGui.QApplication.instance().thread())
+        
+        # Post an event to be dispatched on the main GUI thread. Note that 
+        # we do not call QTimer.singleShot, which would be simpler, because
+        # that only works on QThreads. We want regular Python threads to work.
+        event = QtCore.QEvent(_QT_TRAITS_EVENT)
+        QtGui.QApplication.instance().postEvent(self, event)
 
-        # Dispatch next time round the event loop.
-        QtCore.QTimer.singleShot(0, self._dispatch)
+    def event(self, event):
+        """ QObject event handler. """
+        if event.type() == _QT_TRAITS_EVENT:
+            # Invoke the handler
+            self._handler(*self._args)
 
-    def _dispatch(self):
-        """ Invoke the handler. """
-        # Remove from the list.
+            # We cannot remove from self._calls here. QObjects don't like being 
+            # garbage collected during event handlers (there are tracebacks,
+            # plus maybe a memory leak, I think).
+            QtCore.QTimer.singleShot(0, self._finished)
+
+            return True
+        else:
+            return QtCore.QObject.event(self, event)
+
+    def _finished(self):
+        """ Remove the call from the list, so it can be garbage collected.
+        """
         self._calls_mutex.lock()
         del self._calls[self._calls.index(self)]
         self._calls_mutex.unlock()
 
-        self._handler(*self._args)
-
-
 def ui_handler ( handler, *args ):
     """ Handles UI notification handler requests that occur on a thread other
         than the UI thread.
@@ -343,7 +365,28 @@ class GUIToolkit ( Toolkit ):
         """ Hooks all specified events for all controls in a UI so that they
             can be routed to the correct event handler.
         """
-        pass
+        if events is None:
+            # fixme: Implement drag and drop events ala toolkit.py for wx
+            return
+
+        elif events == 'keys':
+            class KeyEventCatcher(QtCore.QObject):
+                def eventFilter(self, object, event):
+                    if event.type() == QtCore.QEvent.KeyPress:
+                        return handler(event)
+                    else:
+                        return QtCore.QObject.eventFilter(self, object, event)
+            control.installEventFilter(KeyEventCatcher(control))
+
+    #---------------------------------------------------------------------------
+    #  Indicates that an event should continue to be processed by the toolkit
+    #---------------------------------------------------------------------------
+
+    def skip_event ( self, event ):
+        """ Indicates that an event should continue to be processed by the 
+            toolkit.
+        """
+        event.ignore()
 
     #---------------------------------------------------------------------------
     #  Destroys a specified GUI toolkit control:  
@@ -352,20 +395,24 @@ class GUIToolkit ( Toolkit ):
     def destroy_control ( self, control ):
         """ Destroys a specified GUI toolkit control.
         """
-        # Treating dialogs differently seems to be necessary on Windows to
-        # prevent a crash.  I think the problem is that this is being called
-        # from within the finished() signal handler so we need to do the
-        # delete after the handler has returned.
+        # Make sure that any dialogs started with 'exec' are terminated,
+        # otherwise their local event loops will continue to run.
         if isinstance(control, QtGui.QDialog):
-            control.deleteLater()
+            # Block signals to prevent any 'finished()' signals being emitted.
+            # At this point, all cleanup is assumed to be done.
+            control.blockSignals(True)
+            QtGui.QDialog.done(control, 0)
+            control.blockSignals(False)
 
-            # PyQt v4.3.1 and earlier deleteLater() didn't transfer ownership
-            # to C++ which turns out to be important.
-            if QtCore.PYQT_VERSION < 0x040302:
-                import sip
-                sip.transferto(control, None)
-        else:
-            control.setParent(None)
+        # This may be called from within the finished() signal handler
+        # so we need to do the delete after the handler has returned.
+        control.deleteLater()
+
+        # PyQt v4.3.1 and earlier deleteLater() didn't transfer ownership
+        # to C++ which turns out to be important.
+        if QtCore.PYQT_VERSION < 0x040302:
+            import sip
+            sip.transferto(control, None)
 
     #---------------------------------------------------------------------------
     #  Destroys all of the child controls of a specified GUI toolkit control:
@@ -399,7 +446,8 @@ class GUIToolkit ( Toolkit ):
         """ Returns a dictionary of useful constants.
         
             Currently, the dictionary should have the following key/value pairs:
-            - WindowColor': the standard window background color in the toolkit
+                
+            - 'WindowColor': the standard window background color in the toolkit
               specific color format.
         """
         return {
