@@ -2,8 +2,9 @@
 # Copyright (c) 2007, Riverbank Computing Limited
 # All rights reserved.
 #
-# This software is provided without warranty under the terms of the GPL v2
-# license.
+# This software is provided without warranty under the terms of the BSD license.
+# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply
+
 #
 # Author: Riverbank Computing Limited
 #------------------------------------------------------------------------------
@@ -190,6 +191,7 @@ class BasePanel(object):
             name = action.name
         id     = action.id
         button = bbox.addButton(name, role)
+        button.setAutoDefault(False)
         button.setEnabled(enabled)
         if (method is None) or (action.enabled_when != '') or (id != ''):
             editor = ButtonEditor( ui      = ui,
@@ -270,7 +272,10 @@ class _StickyDialog(QtGui.QDialog):
         layout.addWidget(self._mw)
         self.setLayout(layout)
 
-        if not ui.view.resizable:
+        if ui.view.resizable:
+            self.setWindowFlags(QtCore.Qt.WindowMinimizeButtonHint |
+                                QtCore.Qt.WindowMaximizeButtonHint)
+        else:
             layout.setSizeConstraint(QtGui.QLayout.SetFixedSize)
 
         self._ui = ui
@@ -288,11 +293,14 @@ class _StickyDialog(QtGui.QDialog):
             e.ignore()
 
     def keyPressEvent(self, e):
-        """Reimplemented to ignore the Escape key if appropriate."""
+        """Reimplemented to ignore the Escape key if appropriate, and always
+        ignore the Enter key because it will trigger a button if any are
+        present in the dialog."""
 
-        if (e.modifiers() != QtCore.Qt.NoModifier or
-                e.key() != QtCore.Qt.Key_Escape or self._ok_to_close()):
-            QtGui.QDialog.keyPressEvent(self, e)
+        if (e.key() == QtCore.Qt.Key_Enter or e.key() == QtCore.Qt.Key_Return or
+                (e.key() == QtCore.Qt.Key_Escape and not self._ok_to_close())):
+            return
+        QtGui.QDialog.keyPressEvent(self, e)
 
     def done(self, r):
         """Reimplemented to ignore calls to accept() or reject() if
@@ -373,7 +381,7 @@ class BaseDialog(BasePanel):
         self._add_toolbar()
         self._add_statusbar()
 
-    def close(self, rc):
+    def close(self, rc=True):
         """Close the dialog and set the given return code."""
 
         self.ui.dispose(rc)
@@ -430,7 +438,7 @@ class BaseDialog(BasePanel):
         menubar = self.ui.view.menubar
         if menubar is not None:
             self._last_group = self._last_parent = None
-            self.control.SetMenuBar(
+            self.control.layout().setMenuBar(
                 menubar.create_menu_bar( self.control, self ) )
             self._last_group = self._last_parent = None
 
@@ -444,7 +452,7 @@ class BaseDialog(BasePanel):
         toolbar = self.ui.view.toolbar
         if toolbar is not None:
             self._last_group = self._last_parent = None
-            # FIXME: A QDialog has no toolbar. This should e fixed, but right 
+            # FIXME: A QDialog has no toolbar. This should be fixed, but right 
             # now we are passing rather than creating a traceback.
             return 
             self.control.SetToolBar(
@@ -458,6 +466,9 @@ class BaseDialog(BasePanel):
     def _add_statusbar ( self ):
         """ Adds a statusbar to the dialog.
         """
+        # FIXME: We should implement this, but right now we are passing rather
+        # than creating a traceback.
+        return
         statusbar = self.ui.view.statusbar
         if statusbar is not None:
             raise NotImplementedError
