@@ -2,8 +2,9 @@
 # Copyright (c) 2007, Riverbank Computing Limited
 # All rights reserved.
 #
-# This software is provided without warranty under the terms of the GPL v2
-# license.
+# This software is provided without warranty under the terms of the BSD license.
+# However, when used with the GPL version of PyQt the additional terms described
+# in the PyQt GPL exception also apply.
 #
 # Author: Riverbank Computing Limited
 #------------------------------------------------------------------------------
@@ -140,9 +141,10 @@ class _Panel(BasePanel):
         # of an "outer" widget (eg. a dock widget).  These tests are extremely
         # flakey.
         title = view.title
-        if title != "":
-            if isinstance(parent, QtGui.QMainWindow) and not isinstance(parent.parent(), QtGui.QDialog):
-                title = ""
+        if ((isinstance(parent, QtGui.QMainWindow) and 
+             not isinstance(parent.parent(), QtGui.QDialog)) or
+            isinstance(parent, QtGui.QTabWidget)):
+            title = ""
 
         # Panels must be widgets as it is only the TraitsUI PyQt code that can
         # handle them being layouts as well.  Therefore create a widget if the
@@ -163,36 +165,43 @@ class _Panel(BasePanel):
 
             self.control = w
 
-            # FIXME: I do not know how to fix the following code for Qt.
-            # It is preferable to fail gracefully rather than to raise a
-            # useless exception.
-
             # Add any buttons.
-#            if has_buttons:
-#                # Add the special function buttons:
-#                layout.Add( wx.StaticLine( cpanel, -1 ), 0, wx.EXPAND )
-#                b_sizer = wx.BoxSizer( wx.HORIZONTAL )
-#                for button in buttons:
-#                    if self.is_button( button, 'Undo' ):
-#                        self.undo = self.add_button( button, b_sizer, 
-#                                                 self._on_undo, False )
-#                        self.redo = self.add_button( button, b_sizer, 
-#                                                 self._on_redo, False, 'Redo' )
-#                        history.on_trait_change( self._on_undoable, 'undoable',
-#                                             dispatch = 'ui' )
-#                        history.on_trait_change( self._on_redoable, 'redoable',
-#                                             dispatch = 'ui' )
-#                    elif self.is_button( button, 'Revert' ):
-#                        self.revert = self.add_button( button, b_sizer, 
-#                                                   self._on_revert, False )
-#                        history.on_trait_change( self._on_revertable, 'undoable',
-#                                             dispatch = 'ui' )
-#                    elif self.is_button( button, 'Help' ):
-#                        self.add_button( button, b_sizer, self._on_help )
-#                    elif not self.is_button( button, '' ):
-#                        self.add_button( button, b_sizer )
-#
-#                layout.Add( b_sizer, 0, wx.ALIGN_RIGHT | wx.ALL, 5 )
+            if has_buttons:
+
+                # Add the horizontal separator
+                separator = QtGui.QFrame()
+                separator.setFrameStyle(QtGui.QFrame.Sunken | 
+                                        QtGui.QFrame.HLine)
+                separator.setFixedHeight(2)
+                layout.addWidget(separator)
+
+                # Add the special function buttons
+                bbox = QtGui.QDialogButtonBox(QtCore.Qt.Horizontal)
+                for button in buttons:
+                    role = QtGui.QDialogButtonBox.ActionRole
+                    if self.is_button(button, 'Undo'):
+                        self.undo = self.add_button(button, bbox, role,
+                                                    self._on_undo, False,
+                                                    'Undo')
+                        self.redo = self.add_button(button, bbox, role,
+                                                    self._on_redo, False, 
+                                                    'Redo')
+                        history.on_trait_change(self._on_undoable, 'undoable',
+                                                dispatch = 'ui')
+                        history.on_trait_change(self._on_redoable, 'redoable',
+                                                dispatch = 'ui')
+                    elif self.is_button(button, 'Revert'):
+                        role = QtGui.QDialogButtonBox.ResetRole
+                        self.revert = self.add_button(button, bbox, role,
+                                                      self._on_revert, False)
+                        history.on_trait_change(self._on_revertable, 'undoable',
+                                                dispatch = 'ui')
+                    elif self.is_button(button, 'Help'):
+                        role = QtGui.QDialogButtonBox.HelpRole
+                        self.add_button(button, bbox, role, self._on_help)
+                    elif not self.is_button(button, ''):
+                        self.add_button(button, bbox, role)
+                layout.addWidget(bbox)
 
 
 def panel(ui):
@@ -254,7 +263,6 @@ def _fill_panel(panel, content, ui, item_handler=None):
             # down into just the page.
             if type(sub_page) is type(panel) and sub_page.count() == 1:
                 new = sub_page.widget(0)
-
                 if isinstance(panel, QtGui.QTabWidget):
                     sub_page.removeTab(0)
                 else:
@@ -265,7 +273,9 @@ def _fill_panel(panel, content, ui, item_handler=None):
                 new = QtGui.QWidget()
                 new.setLayout(page)
 
-            new.layout().setAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
+            layout = new.layout()
+            if layout is not None:
+                layout.setAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
 
             # Add the content.
             if isinstance(panel, QtGui.QTabWidget):
@@ -392,6 +402,13 @@ class _GroupPanel(object):
             # Create the splitter.
             splitter = QtGui.QSplitter()
 
+            size_policy = splitter.sizePolicy()
+            if group.orientation == 'horizontal':
+                size_policy.setVerticalPolicy(QtGui.QSizePolicy.Expanding)
+            else:
+                size_policy.setHorizontalPolicy(QtGui.QSizePolicy.Expanding)
+            splitter.setSizePolicy(size_policy)
+
             if self.direction == QtGui.QBoxLayout.TopToBottom:
                 splitter.setOrientation(QtCore.Qt.Vertical)
 
@@ -402,7 +419,8 @@ class _GroupPanel(object):
 
             # Create an editor.
             self._setup_editor(group,
-                    SplitterGroupEditor(control=outer, splitter=splitter))
+                    SplitterGroupEditor(control=outer, splitter=splitter,
+                                        ui=ui))
 
             self._add_splitter_items(content, splitter)
 
@@ -436,6 +454,7 @@ class _GroupPanel(object):
                 layout = self._add_groups(content, inner)
             else:
                 layout = self._add_items(content, inner)
+            layout.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop)
 
             if outer is None:
                 outer = layout
@@ -453,17 +472,28 @@ class _GroupPanel(object):
         """
         use_widths = (splitter.orientation() == QtCore.Qt.Horizontal)
         sizes = []
-
         for item in content:
-            if use_widths:
-                sizes.append(item.width)
-            else:
-                sizes.append(item.height)
 
+            # Get a size for the Item or Group
             if isinstance(item, Group):
                 panel = _GroupPanel(item, self.ui, suppress_label=True).control
+
+                def max_child_size(item):
+                    if isinstance(item, Group):
+                        return max(map(max_child_size, item.content))
+                    else:
+                        if use_widths:
+                            return item.width
+                        else:
+                            return item.height
+
+                sizes.append(max_child_size(item))
             else:
                 panel = self._add_items([item])
+                if use_widths:
+                    sizes.append(item.width)
+                else:
+                    sizes.append(item.height)
 
             # Add the panel to the splitter.
             if panel is not None:
@@ -474,7 +504,10 @@ class _GroupPanel(object):
                     w.setLayout(panel)
                     panel = w
 
-                panel.layout().setAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
+                layout = panel.layout()
+                if layout is not None:
+                    layout.setAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
+
                 splitter.addWidget(panel)
 
         # Find out how much space is available.
@@ -487,7 +520,6 @@ class _GroupPanel(object):
         # Allocate requested space.
         avail = total
         remain = 0
-
         for i, sz in enumerate(sizes):
             if avail <= 0:
                 break
@@ -633,7 +665,10 @@ class _GroupPanel(object):
                     else:
                         label = heading_text(None, text=label).control
 
-                    inner.addWidget(label, row, col)
+                    if row < 0:
+                        inner.addWidget(label)
+                    else:
+                        inner.addWidget(label, row, col)
 
                     if item.emphasized:
                         self._add_emphasis(label)
@@ -867,6 +902,10 @@ class _GroupPanel(object):
                 layout.addLayout(w)
 
         else:
+            if self.direction == QtGui.QBoxLayout.LeftToRight:
+                # Flip the row and column.
+                row, column = column, row
+
             if show_labels:
                 # Convert the "logical" column to a "physical" one.
                 column *= 2
@@ -875,10 +914,6 @@ class _GroupPanel(object):
                    (label_alignment == 0 and self.group.show_left):
                     column += 1
 
-            if self.direction == QtGui.QBoxLayout.LeftToRight:
-                # Flip the row and column.
-                row, column = column, row
-
             if isinstance(w, QtGui.QWidget):
                 layout.addWidget(w, row, column, label_alignment)
             else:
